"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const path = require("path");
const fs = require("mz/fs");
const mkdirp_1 = require("../fs/mkdirp");
const requireJson_1 = require("../fs/requireJson");
const getPkgDirs_1 = require("../fs/getPkgDirs");
const binify_1 = require("../binify");
const isWindows = require("is-windows");
const cmdShim = require("@zkochan/cmd-shim");
const pnpm_logger_1 = require("pnpm-logger");
const IS_WINDOWS = isWindows();
function linkAllBins(modules, binPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkgDirs = yield getPkgDirs_1.default(modules);
        return Promise.all(pkgDirs.map((pkgDir) => linkPkgBins(pkgDir, binPath)));
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = linkAllBins;
/**
 * Links executable into `node_modules/.bin`.
 */
function linkPkgBins(target, binPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkg = yield safeRequireJson(path.join(target, 'package.json'));
        const targetRealPath = yield fs.realpath(target);
        const extraNodePath = [path.join(targetRealPath, 'node_modules'), path.join(targetRealPath, '..', 'node_modules')];
        if (!pkg) {
            pnpm_logger_1.default.warn(`There's a directory in node_modules without package.json: ${target}`);
            return;
        }
        if (!pkg.bin)
            return;
        const bins = binify_1.default(pkg);
        yield mkdirp_1.default(binPath);
        yield Promise.all(Object.keys(bins).map(function (bin) {
            return __awaiter(this, void 0, void 0, function* () {
                const externalBinPath = path.join(binPath, bin);
                const actualBin = bins[bin];
                const targetPath = path.join(target, actualBin);
                const nodePath = extraNodePath.concat(getNodePaths(targetPath)).join(path.delimiter);
                return cmdShim(targetPath, externalBinPath, { nodePath });
            });
        }));
    });
}
exports.linkPkgBins = linkPkgBins;
function getNodePaths(filename) {
    const next = path.join(filename, '..');
    const modules = path.join(filename, 'node_modules');
    if (filename === next)
        return [modules];
    return [modules].concat(getNodePaths(next));
}
/**
 * Like `require()`, but returns `null` when it is not found
 */
function safeRequireJson(pkgJsonPath) {
    try {
        return requireJson_1.default(pkgJsonPath);
    }
    catch (err) {
        if (err.code !== 'ENOENT')
            throw err;
        return null;
    }
}
//# sourceMappingURL=linkBins.js.map